# -*- coding: utf-8 -*-

# inflation.py - Inflation computations and class
# Copyright (C) 2013 Tryggvi Björgvinsson
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import datetime
import collections
import economics

InflationResult = collections.namedtuple('Inflation', 'factor value')

class Inflation(object):
    """
    Object to provide simple inflation computational functions
    """

    def __init__(self, source=economics.CPI, reference=None, country=None):
        """
        Create a new Inflation instance using the CPI class. Optional
        parameters are source which defaults to economics.CPI class, 
        reference date (datetime.date), country (name or code).
        """
        
        # Create a new data source from the source
        self.data = source(country=country)

        # Set reference and country based on parameters
        self.reference = reference

    def _compute_inflation(self, value, reference_value):
        """
        Helper function to compute the inflation/deflation based on a value and
        a reference value
        """
        return InflationResult(
            factor=value / float(reference_value),
            value=(value - reference_value) / float(reference_value))

    def get(self, target=datetime.date.today(), reference=None, country=None):
        """
        Get the inflation/deflation value change for the target date based 
        on the reference date. Target defaults to today and the instance's
        reference and country will be used if they are not provided as
        parameters
        """

        # Set country & reference to object's country & reference respectively
        reference = self.reference if reference is None else reference

        # Get the reference and target indices (values) from the source
        reference_value = self.data.get(reference, country).value
        target_value = self.data.get(target, country).value

        # Compute the inflation value and return it
        return self._compute_inflation(target_value, reference_value)

    def inflate(self, amount, target=datetime.date.today(), reference=None,
                country=None):
        """
        Inflate a given amount to the target date from a reference date (or
        the object's reference year if no reference date is provided) in a
        given country (or objects country if no country is provided). The
        amount has to be provided as it was valued in the reference year.
        """

        # Get the inflation for the two dates and country
        inflation = self.get(target, reference, country)
        # Return the inflated/deflated amount
        return amount * inflation.factor



# -*- coding: utf-8 -*-

# datastructures.py - Various helpful datastructures or converters
# Copyright (C) 2013 Tryggvi Björgvinsson
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import time

class MapDict(dict):
    """
    Dictionary that allows for multiple keys to the same value
    """

    def __init__(self, mapping={}):
        """
        Initialise the MapDict by creating a key map and interate through a
        provided mapping (or an empty dict if not present)
        """
        self.map = {}
        for k,v in mapping.iteritems():
            self.__setitem__(k,v)
        super(MapDict,self).__init__()

    def __getitem__(self, key):
        """
        Get a value for a key (can be in the key map or the key itself)
        """
        return super(MapDict,self).__getitem__(self.map.get(key, key))
        
    def __setitem__(self, keys, value):
        """
        Set a value for keys. In case keys have an iterator first value is set
        as the main key, the rest are mapped. If the main key is already in the
        key map it is removed from it before proceeding.
        """
        if hasattr(keys, '__iter__'):
            if keys[0] in self.map:
                del self.map[keys[0]]
            for key in keys[1:]:
                self.map[key] = keys[0]
            super(MapDict,self).__setitem__(keys[0],value)
        else:
            super(MapDict,self).__setitem__(keys,value)
        
    def __delitem__(self, key):
        """
        Remove a key and value (along with all mapped keys)
        """
        mappedkey = self.map.get(key, key)
        for k,v in self.map.items():
            if v == mappedkey:
                del self.map[k]
        super(MapDict,self).__delitem__(mappedkey)

    def update(self, mapdict):
        """
        Update the MapDict with another MapDict (update the key map as well)
        """
        self.map.update(mapdict.map)
        return super(MapDict,self).update(mapdict)




# -*- coding: utf-8 -*-

# cpi.py - Consumer Price Index data manipulation, computation and Class
# Copyright (C) 2013 Tryggvi Björgvinsson
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import datetime
import collections
import data
from datastructures import MapDict

CPIResult = collections.namedtuple('CPI', 'date value')

class CPI(object):
    """
    Provides a Pythonic interface to Consumer Price Index data packages
    """

    def __init__(self, datapackage='http://data.okfn.org/data/cpi/',
                 country=None):
        """
        Initialise a CPI instance. Default data package location is the cpi
        data on http://data.okfn.org
        """

        # Store datapackage and country as instance variables
        self.datapackage = datapackage
        self.country = country

        # Initialise empty data structures
        self.data = MapDict()

        # Load the data into the data structures
        self.load()

    def load(self):
        """
        Load data with the data from the datapackage
        """

        # Loop through the rows of the datapackage with the help of data
        for row in data.get(self.datapackage):
            # Get the code and the name and transform to uppercase
            # so that it'll match no matter the case
            code = row['Country Code'].upper()
            name = row['Country Name'].upper()
            # Get the date (which is in the field Year) and the CPI value
            date = row['Year']
            cpi = row['CPI']
            
            # Try to get the data for country
            # or initialise an empty dict
            country_data = self.data.get(code, {})
            # Set the CPI value for the date
            country_data[date] = cpi

            # Set the code as the default key and name as extra
            # key in the mapdict for the country data
            self.data[(code, name)] = country_data

    def get(self, date=datetime.date.today(), country=None):
        """
        Get the CPI value for a specific time. Defaults to today. This uses
        the closest method internally but sets limit to one day.
        """
        try:
            return CPIResult(date=date, value=self.data[country.upper()][date])
        except:
            raise KeyError('Date {date} not found in data'.format(date=date))

    def closest(self, date=datetime.date.today(), country=None,
                limit=datetime.timedelta(days=366)):
        """
        Get the closest CPI value for a specified date. The date defaults to
        today. A limit can be provided to exclude all values for dates further
        away than defined by the limit. This defaults to 366 days.
        """

        # Try to get the country
        country = self.country if country is None else country

        # Get the data for the given country (we store it as uppercase)
        country_data = self.data[country.upper()]

        # Find the closest date
        closest_date = min(country_data.keys(),
                           key=lambda x: abs(date-x))
        
        # We return the CPI value if it's within the limit or raise an error
        if abs(date-closest_date) < limit:
            return CPIResult(date=closest_date,
                             value=country_data[closest_date])
        else:
            raise KeyError('A date close enough was not found in data')



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup

description = 'Functions and data manipulation for economics data'
with open('README.rst') as readme:
    long_description = readme.read()

setup(
    name = 'economics',
    version = '0.1.2',
    url = 'https://github.com/tryggvib/economics',
    license = 'GPLv3',
    description = description,
    long_description = long_description,
    author = 'Tryggvi Björgvinsson',
    author_email = 'tryggvi.bjorgvinsson@okfn.org',
    install_requires = ['datapackage>=0.4.1'],
    packages = ['economics'],
    package_dir={'economics': 'economics'},
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Intended Audience :: Financial and Insurance Industry',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Office/Business :: Financial',
        'Topic :: Utilities',
    ],
)
